#!/usr/bin/env python
import math
import sys
import os
import numpy as np
from config import *
import matplotlib
import matplotlib.pyplot as plt


def draw_fig(dstr, mtds, coreslist, C, epss, loss):
    fig = plt.figure()
    axe = fig.add_subplot(111)
    plt.tight_layout()
    cmap = plt.get_cmap('hsv')
    
    minimal = dobj[loss][dstr]
    makr = ["--", "-", "ko-."]
    #makr = ["^--", "*-", "o-."]
    
    LIM = dlim[loss][dstr]
    LIM_DIFF = dstart[loss][dstr]
   
    totnum = sum([len(li) for li in coreslist])
    colors = [cmap(j) for j in np.linspace(0, 1, totnum+1)]
    clridx = 0

    for (i, mtd) in enumerate(mtds):
        for core in coreslist[i]:
            vals = []
            tms = []
            logname = ".".join([dstr, mtd, "n%d" % (core), "c%s" % (C), "e%s" % (epss[i]), "log"])
            print(logname) 
            try:
                fl = open(LOGPATH+logname, "r")
            except IOError:
                continue

            for line in fl:
                line = line.split(' ')
                if 'd' in line and 'walltime' in line:
                    iternum = int(line[1])
                    vals.append(float(line[line.index('d')+1]))
                    tms.append(float(line[line.index('walltime')+1]))
        
            relvals = [math.fabs( (obj-minimal)/minimal ) for obj in vals] 
            #print(relvals)
            axe.plot(tms, relvals, makr[i], color=colors[clridx], label=mtd+"-"+str(core), linewidth=3, markersize=6, markevery=0.1)
            clridx += 1
    
    #axe.set_title(dstr)
    axe.set_yscale('log')
    axe.set_ylim(LIM_DIFF)
    #axe.set_yticks(np.linspace(1e-5, 1e-2, 4))
    axe.set_xlim(0, LIM)
    axe.legend()

    plt.show()
    return fig

def traverse(path):
    dset = []
    for root, dirs, files in os.walk(path):
        for fl in files:
            if "ori" in fl:
                dset.append(fl[:fl.find("ori")-1])
    return dset

if len(sys.argv) != 5:
    print("Usage: python plot.py [LOGPATH] [C] [eps] [loss]")
    exit()


LOGPATH = sys.argv[1]
print(LOGPATH)
traverse(LOGPATH)
dataset = traverse(LOGPATH)

C = sys.argv[2]
eps = sys.argv[3]
loss = sys.argv[4]

matplotlib.rc('xtick', labelsize=20)
matplotlib.rc('ytick', labelsize=20)

for dstr in dataset:
    print(dstr)
    corelist = [[1, 2, 4], [1, 2, 4], [1]]
    fig = draw_fig(dstr, ["pass", "eps5new", "ori"], corelist, C, [eps, eps, eps], loss)
    fig.savefig(dstr+"_%s.png" % ("filename"))
